@@ -11,13 +11,13 @@ import os
 import time
 from functools import partial
 from os import mkdir, rmdir
+from queue import Queue
 
 from watchdog.events import FileSystemEventHandler
 from watchdog.observers import Observer
 from watchdog.observers.api import ObservedWatch
 from watchdog.observers.fsevents import FSEventsEmitter
 
-from . import Queue
 from .shell import mkdtemp, rm, touch
 
 logging.basicConfig(level=logging.DEBUG)
